@@ -105,10 +105,16 @@ static void z_neg(element_ptr n, element_ptr a)
     mpz_neg(n->data, a->data);
 }
 
-//only correct for 1, -1
 static void z_invert(element_ptr n, element_ptr a)
 {
-    mpz_set(n->data, a->data);
+    if (!mpz_cmpabs_ui(a->data, 1)) {
+	mpz_set(n->data, a->data);
+    } else mpz_set_ui(n->data, 0);
+}
+
+static void z_div(element_ptr c, element_ptr a, element_ptr b)
+{
+    mpz_tdiv_q(c->data, a->data, b->data);
 }
 
 //(doesn't make sense if order is infinite)
@@ -117,8 +123,7 @@ static void z_random(element_ptr n)
     mpz_set_ui(n->data, 0);
 }
 
-static void z_from_hash(element_ptr n, int len, void *data)
-    //TODO: something more sophisticated!
+static void z_from_hash(element_ptr n, void *data, int len)
 {
     mpz_import(n->data, len, -1, 1, -1, 0, data);
 }
@@ -227,7 +232,7 @@ static int z_length_in_bytes(element_ptr a)
     return (mpz_sizeinbase(a->data, 2) + 7) / 8 + 4;
 }
 
-static void z_print_info(FILE *out, field_ptr f)
+static void z_out_info(FILE *out, field_ptr f)
 {
     UNUSED_VAR(f);
     fprintf(out, "Z: wrapped GMP\n");
@@ -255,6 +260,7 @@ void field_init_z(field_ptr f)
     f->neg = z_neg;
     f->cmp = z_cmp;
     f->invert = z_invert;
+    f->div = z_div;
     f->random = z_random;
     f->from_hash = z_from_hash;
     f->is1 = z_is1;
@@ -269,7 +275,7 @@ void field_init_z(field_ptr f)
     f->to_mpz = z_to_mpz;
     f->length_in_bytes = z_length_in_bytes;
 
-    f->print_info = z_print_info;
+    f->out_info = z_out_info;
 
     mpz_set_ui(f->order, 0);
     f->data = NULL;
